
from __future__ import unicode_literals

__version__ = "2.1"
__project__ = "pyrqlite"

__author__ = "Zac Medico"
__email__ = "zmedico@gmail.com"

__copyright__ = "Copyright (C) 2016 Zac Medico"
__license__ = "MIT"
__description__ = "Python dbapi2 driver for rqlite"

UNLIMITED_REDIRECTS = -1
